#include "oswald-ui.h"
#include "oswald_strings.h"
#include "Fonts.h"

#include "LcdDisplay.h"

#define NUM_LCD_ROWS  96
#define NUM_LCD_COL_BYTES  ( 12 )
#define MAX_FONT_ROWS ( 19 )


void DrawLcdLineBresenham(u8t xstart, u8t ystart, u8t xend, u8t yend)
{
	int x, y, t, dx, dy, incx, incy, pdx, pdy, ddx, ddy, es, el, err;
 
	dx = xend - xstart;
	dy = yend - ystart;
 
	incx = (dx >= 0) ? 1 : -1;
	incy = (dy >= 0) ? 1 : -1;

	if (dx<0)
		dx = -dx;
	if (dy<0)
		dy = -dy;
 
	if (dx>dy) {
		pdx = incx; pdy = 0;
		ddx=incx; ddy=incy;
		es =dy;   el =dx;
	} else {
		pdx=0;    pdy=incy;
		ddx=incx; ddy=incy;
		es =dx;   el =dy;
	}
 
	x = xstart;
	y = ystart;
	err = el/2;
	lcd_set_pixel(x, y, TRUE);
 
	for (t = 0; t < el; ++t) {
		err -= es; 
		if (err < 0) {
			err += el;
			x += ddx;
			y += ddy;
		} else {
			x += pdx;
			y += pdy;
		}
		lcd_set_pixel(x, y, TRUE);
	}
}

void DrawLcdLineBresenhamWW(u8t xstart, u8t ystart, u8t xend, u8t yend, u8t thickness)
{
	int i, x, y, t, dx, dy, incx, incy, pdx, pdy, ddx, ddy, es, el, err;
 
	dx = xend - xstart;
	dy = yend - ystart;
 
	incx = (dx >= 0) ? 1 : -1;
	incy = (dy >= 0) ? 1 : -1;

	if (dx<0)
		dx = -dx;
	if (dy<0)
		dy = -dy;
 
	if (dx>dy) {
		pdx = incx; pdy = 0;
		ddx=incx; ddy=incy;
		es =dy;   el =dx;
	} else {
		pdx=0;    pdy=incy;
		ddx=incx; ddy=incy;
		es =dx;   el =dy;
	}
 
	x = xstart;
	y = ystart;
	err = el/2;
	lcd_set_pixel(x, y, TRUE);
	for (i=1; i<thickness; i++) {
		lcd_set_pixel(x-i, y, TRUE);
		lcd_set_pixel(x+i, y, TRUE);
		lcd_set_pixel(x, y-i, TRUE);
		lcd_set_pixel(x, y+i, TRUE);
	}
 
	for (t = 0; t < el; ++t) {
		err -= es; 
		if (err < 0) {
			err += el;
			x += ddx;
			y += ddy;
		} else {
			x += pdx;
			y += pdy;
		}
		lcd_set_pixel(x, y, TRUE);
		for (i=1; i<thickness; i++) {
			lcd_set_pixel(x-i, y, TRUE);
			lcd_set_pixel(x+i, y, TRUE);
			lcd_set_pixel(x, y-i, TRUE);
			lcd_set_pixel(x, y+i, TRUE);
		}
	}
}


u8t WriteLcdCharacter(u8t x, u8t y, u8t Character)
{
	u8t CharacterHeight = GetCharacterHeight();
	u8t CharacterWidth = GetCharacterWidth(Character);
	unsigned int bitmap[MAX_FONT_ROWS];
	register lx, ly;

	GetCharacterBitmap(Character,(unsigned int*)&bitmap);

	// printf("cw=%d ch=%d\n", CharacterWidth, CharacterHeight);
	for (ly=0; ly<CharacterHeight; ly++) {
		for (lx=0; lx<CharacterWidth; lx++) {
			if (bitmap[ly] & (1<<lx)) {
				lcd_set_pixel(lx+x, ly+y, TRUE);
				// printf(".");
			} else {
				lcd_set_pixel(lx+x, ly+y, FALSE);
				// printf(" ");
			}
		}
		// printf("\n");
	}

	return CharacterWidth + GetFontSpacing();
}

void WriteLcdString(u8t x, u8t y, u8t *str)
{
	register lx, i, strl;

	strl = oswald_strlen(str);
	if (strl == 0)
		return;

	lx = x;
	for (i=0; i<strl; i++) {
		lx += WriteLcdCharacter(lx, y, str[i]);
	}
}

